<?PHP

/*
 * Copyright (c) 2008 Chris Smith
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

class Discoverer {

 private $server = null;
 private $delegate = '';
 private $identity = '';

 public function __construct($uri) {
  $this->discover($this->identity = $this->normalise($uri));
 }

 public function getServer() {
  return $this->server;
 }

 public function getDelegate() {
  return $this->delegate;
 }

 public function getIdentity() {
  return $this->identity;
 }

 private function normalise($uri) {
  // Strip xri:// prefix
  if (substr($uri, 0, 6) == 'xri://') {
   $uri = substr($uri, 6);
  }

  // If the first char is a global context symbol, treat it as XRI
  if (in_array($uri[0], array('=', '@', '+', '$', '!'))) {
   // TODO: Implement
   throw new Exception('This implementation does not currently support XRI');
  }

  // Add http:// if needed
  if (substr($uri, 0, 7) != 'http://' && substr($uri, 0, 8) != 'https://') {
   $uri = 'http://' . $uri;
  }

  // Strip any fragments
  if (($frag = strpos($uri, '#')) !== false) {
   $uri = substr($uri, 0, $frag);
  }

  return $uri;
 }

 private function discover($uri) {
  // TODO: Yaris discovery

  $this->delegate = $uri;
  $this->server = null;

  $this->htmlDiscover($uri);
 }

 private function htmlDiscover($uri) {
  $data = @file_get_contents($uri);
  
  preg_match_all('#<link\s*(.*?)\s*/?>#is', $data, $matches);

  foreach ($matches[1] as $link) {
   $rel = $href = null;

   if (preg_match('#rel\s*=\s*(?:([^"\'>\s]*)|"([^">]*)"|\'([^\'>]*)\')(?:\s|$)#is', $link, $m)) {
    $rel = $m[1] . $m[2] . $m[3]; 
   }

   if (preg_match('#href\s*=\s*(?:([^"\'>\s]*)|"([^">]*)"|\'([^\'>]*)\')(?:\s|$)#is', $link, $m)) {
    $href = $m[1] . $m[2] . $m[3];
   }

   if ($rel == 'openid.server') {
    $this->server = $href;
   } else if ($rel == 'openid.delegate') {
    $this->delegate = $href;
   }
  }
 }

}

?>
