<?PHP

/*
 * Copyright (c) 2008 Chris Smith
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

 require_once(dirname(__FILE__) . '/discoverer.inc.php');
 require_once(dirname(__FILE__) . '/poster.inc.php');
 require_once(dirname(__FILE__) . '/urlbuilder.inc.php');

 function redirect() {
  if (defined('REDIRECT_URL')) {
   $url = REDIRECT_URL;
  } else if (isset($_SESSION['openid']['redirect'])) {
   $url = $_SESSION['openid']['redirect'];
  } else {
   $url = UrlBuilder::getCurrentURL();
   $url = substr($url, 0, ($p = strpos($url, '?')) === false ? strlen($url) : $p);
  }

  doRedirect($url);
 }

 function doRedirect($url) {
  header('Location: ' . $url);
  echo '<html><head><title>Redirecting</title></head><body>';
  echo '<p>Redirecting to <a href="', htmlentities($url), '">';
  echo htmlentities($url), '</a></p></body></html>';
  exit();
 }

 if (session_id() == "") {
  // No session - testing maybe?
  session_start();
 }

 if (isset($_REQUEST['openid_url'])) {
  $disc = new Discoverer($_POST['openid_url']);

  $_SESSION['openid'] = array(
 	'identity' => $disc->getIdentity(), 
	'delegate' => $disc->getDelegate(),
	'validated' => false,
	'server' => $disc->getServer()
  );

  if ($disc->getServer() == null) {
   $_SESSION['openid']['error'] = 'Claimed identity is not a valid identifier';
   redirect();
  }

  $url = URLBuilder::buildImmediate($disc->getServer(), $disc->getDelegate(), URLBuilder::getCurrentURL());

  doRedirect($url);
 } else if (isset($_REQUEST['openid_mode'])) {
  switch ($_REQUEST['openid_mode']) {
   case 'id_res':
    if (isset($_REQUEST['openid_identity'])) {
     $url = URLBuilder::buildAuth($_REQUEST);
     $data = Poster::post($_SESSION['openid']['server'], $url);
     foreach (explode("\n", $data) as $line) {
      if (substr($line, 0, 9) == 'is_valid:') {
       $valid = (boolean) substr($line, 9);
      }
     }

     $_SESSION['openid']['validated'] = $valid;

     if (!$valid) {
      $_SESSION['openid']['error'] = 'Provider didn\'t authenticate response';
     }

     redirect();

    } else if (isset($_REQUEST['openid_user_setup_url'])) {
     $url = URLBuilder::buildSetup($_REQUEST['openid_user_setup_url'], $_SESSION['openid']['delegate'], URLBuilder::getCurrentURL());
     doRedirect($url);
    }
    break;
   case 'cancel':
    $_SESSION['openid']['error'] = 'Provider cancelled the authentication attempt';

    redirect();
    break; 
   case 'error':
    $_SESSION['openid']['error'] = 'Provider error: ' . $_REQUEST['openid_error'];

    redirect();
    break;
  }
 }


?>
